#ifndef __copy_factory
#define __copy_factory

//------------------------------ includes ------------------------------
#include <qobject.h>

// These class declaraion is needed to avoid including all widgets in this header file.
class QCheckBox;
class QLineEdit;
class QButton;
class QRadioButton;
class QButtonGroup;
class QLabel;
class QTabWidget;
class QWidget;
class ExtEntry;
class QMultiLineEdit;
class ExtEntry;
class ExtEntryEntry;
//----------------------------------------------------------------------

/**
   This class defines methods to copy information between widgets of the equal type.
 
   This is necessary when the content of one widget is copied to another,
   as no copy constructor is defined for Qt and KDE widgets.

   With time this class should be expanded to contain every Qt/KDE widgets,
   but until that is fulfilled, the user may extend it himself. He does
   that by inheriting either the @ref ExtEntryFactory or the @ref
   FillOutFactory, and then override their @ref copy method.
**/
class CopyFactory : public QObject
{

  Q_OBJECT

public:
  /**
     Topmost method which the user of this class will call. This method
     will determine the type of the widget to copy, and call the
     appropriate method. If the user wants to add extra widgets it may copy
     (either Qt/KDE widgets or his own personal widgets) then he may
     override this method. In his overridden version he should test if
     `from' is of this extra type, and if that is the case then copy it,
     otherwise call the super classes copy method, like in the following
     code:
     <pre>
     if (dynamic_cast<const MyClass *>(from)) 
       copy_MyClass((MyClass *) from, (MyClass *) to);
     else
       ExtEntryFactory::copy(from,to);
     </pre>
  **/

  virtual void copy(const QObject *from, QObject *to);
  virtual void copy_ExtEntry(const ExtEntry *from,ExtEntry *to);
  virtual void copy_ExtEntryEntry(const ExtEntryEntry *from,ExtEntryEntry *to);
  virtual void copy_QButton(const QButton *from, QButton *to);
  virtual void copy_QRadioButton(const QRadioButton *from, QRadioButton *to);
  virtual void copy_QButtonGroup(const QButtonGroup *from, QButtonGroup *to);
  virtual void copy_QCheckBox(const QCheckBox *from, QCheckBox *to);
  virtual void copy_QLabel(const QLabel *from, QLabel *to);
  virtual void copy_QLineEdit(const QLineEdit *from, QLineEdit *to);
  virtual void copy_QMultiLineEdit(const QMultiLineEdit *from, QMultiLineEdit *to);
  virtual void copy_QTabWidget(const QTabWidget *from, QTabWidget *to);
  virtual void copy_QWidget(const QWidget *from, QWidget *to);
};



#endif /* factory */

