// -------------------------------- includes ------------------------------
#ifndef __factory
#define __factory
#include <qobject.h>
#include "copyfact.h"
//-------------------------------------------------------------------------


/**
   Factory use to generate elements for the @ref ExtEntry widget.

   To use an @ref ExtEntry, one must inherit from the class ExtEntryFactory
   and override the method @ref create. This method must return an empty
   element for the ExtEntry (that is an @ref ExtEntryEntry or a subclass of
   it).

   If you dislike the default separator between each of the elements or
   simply do not want a separator in the ExtEntry, then you may override
   the method @ref separator.
**/
class ExtEntryFactory : public CopyFactory
{
  Q_OBJECT

public:
  /**
     This method must be overridden in subclasses and must return an
     ``empty'' instance of the @ref ExtEntryEntry class, or perhaps rather
     a subclass of this class. This instance will be owned by the caller of
     this function.

     @param parent A pointer to the parent of this ExtEntryEntry widget
     returned.
     @return A fresh @ref ExtEntryEntry to be used in an instance of the
     @ref ExtEntry class.
  **/
  virtual ExtEntryEntry *create(QWidget *parent) = 0;

  /**
     This method is used to get a separator between the elements in an @ref
     ExtEntry. The widget returned from this method will be owned by the
     caller.

     @param parent A pointer to the parent of the QWidget returned.
     @return A widget which must be used as a separator between the @ref
     ExtEntryEntry elements in an @ref ExtEntry.
  **/
  virtual QWidget *separator(QWidget *parent);
};

#endif /* factory */

