#ifndef __filloutfactory
#define __filloutfactory

#include <qobject.h>
#include "filloutelm.h"
#include "copyfact.h"

/**
   Factory generating elements for the @ref FillOut widget.

   This class is used to get elements for the FillOut widget. Each time a
   new element is required the @ref create method is invoked. This method
   must return a freshly made element.

   The elements is available through a popup menu which appears when the
   user presses the right mouse button.

   The number of available elements is reported through the method @ref
   numberOfElements. The text to be inserted into the popup menu for each
   of the possible elements is obtained by calling the @ref menuName
   method.
**/
class FillOutFactory :public CopyFactory {
Q_OBJECT

public:

  /**
     This method must return the number of different elements available from this
     factory.
  **/
  virtual int numberOfElements() = 0; 

  /**
     This method must return a freshly made element for the @ref FillOut
     widget. It is the caller of this function, which owns the new element.

     @param parent A pointer to the parent of the new widget.
     @param which a number indicating which of the possible @ref FillOutElm
     element should be created. `which' should be in the range 0 to
     number-of-element minus 1.
  **/
  virtual FillOutElm *create(QWidget *parent, int which) = 0; 

  /** 
      This function must return a string to be used in the popup menu for
      the element numbered by `which'.
  **/
  virtual QString menuName(int which) = 0;

};
#endif /* filloutfactory */

